@@ -8,7 +8,7 @@ debug_opts="-O2"
 def_opts=
 largefile_opts="-D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE=1"
 fifo_opts=""
-libvers="-DCdrskin_libburn_1_1_6"
+libvers="-DCdrskin_libburn_1_1_8"
 
 # To be used if Makefile.am uses libburn_libburn_la_CFLAGS
 # burn="libburn/libburn_libburn_la-"
@@ -41,31 +41,21 @@ do
   elif test "$i" = "-compile_dewav"
   then
     compile_dewav=1
-  elif test "$i" = "-cvs_A60220"
+  elif test "$i" = "-libburn_1_1_8"
   then
-    libvers="-DCdrskin_libburn_cvs_A60220_tS"
-    libdax_audioxtr_o=
-    libdax_msgs_o="$burn"message.o
-    cleanup_src_or_obj="-DCleanup_has_no_libburn_os_H cdrskin/cleanup.c"
-  elif test "$i" = "-libburn_1_1_6"
-  then
-    libvers="-DCdrskin_libburn_1_1_6"
+    libvers="-DCdrskin_libburn_1_1_8"
     libdax_audioxtr_o="$burn"libdax_audioxtr.o
     libdax_msgs_o="$burn"libdax_msgs.o
     cleanup_src_or_obj="$burn"cleanup.o
   elif test "$i" = "-libburn_svn"
   then
-    libvers="-DCdrskin_libburn_1_1_7"
+    libvers="-DCdrskin_libburn_1_1_9"
     libdax_audioxtr_o="$burn"libdax_audioxtr.o
     libdax_msgs_o="$burn"libdax_msgs.o
     cleanup_src_or_obj="$burn"cleanup.o
   elif test "$i" = "-newapi" -o "$i" = "-experimental"
   then
     def_opts="$def_opts -DCdrskin_new_api_tesT"
-  elif test "$i" = "-oldfashioned"
-  then
-    def_opts="$def_opts -DCdrskin_oldfashioned_api_usE"
-    cleanup_src_or_obj="-DCleanup_has_no_libburn_os_H cdrskin/cleanup.c"
   elif test "$i" = "-no_largefile"
   then
     largefile_opts=	
@@ -109,7 +99,7 @@ do
     echo "Options:"
     echo "  -compile_cdrfifo  compile program cdrskin/cdrfifo."
     echo "  -compile_dewav    compile program test/dewav without libburn."
-    echo "  -libburn_1_1_6    set macro to match libburn-1.1.6"
+    echo "  -libburn_1_1_8    set macro to match libburn-1.1.8"
     echo "  -libburn_svn      set macro to match current libburn-SVN."
     echo "  -dvd_obs_64k      64 KB default size for DVD/BD writing."
     echo "  -use_libcdio      link with -lcdio because libburn uses it."
@@ -117,7 +107,6 @@ do
     echo "  -use_no_libburn_fifo  use cdrfifo even for single track non-CD"
     echo "  -use_no_cdrfifo   always use fifo of libburn and never cdrfifo"
     echo "  -experimental     use newly introduced libburn features."
-    echo "  -oldfashioned     use pre-0.2.2 libburn features only."
     echo "  -do_diet          produce capability reduced lean version."
     echo "  -do_strip         apply program strip to compiled programs."
     echo "  -g                produce debuggable programm."
