@@ -348,7 +348,7 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	struct buffer *buf = NULL;
 	struct scsi_mode_data *m;
 	int page_length, num_write_speeds = 0, i, speed, ret;
-	int old_alloc_len, was_error = 0;
+	int old_alloc_len, was_error = 0, block_descr_len;
 	unsigned char *page;
 	struct command *c = NULL;
 	struct burn_speed_descriptor *sd;
@@ -371,10 +371,6 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 
 	memset(buf, 0, sizeof(struct buffer));
 	scsi_init_command(c, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
-/*
-	memcpy(c->opcode, SPC_MODE_SENSE, sizeof(SPC_MODE_SENSE));
-	c->oplen = sizeof(SPC_MODE_SENSE);
-*/
 	c->dxfer_len = *alloc_len;
 	c->opcode[7] = (c->dxfer_len >> 8) & 0xff;
 	c->opcode[8] = c->dxfer_len & 0xff;
@@ -391,13 +387,22 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 		was_error = 1;
 	}
 
-	page = c->page->data + 8;
+	/* ts B11103 : qemu SCSI CD-ROM has Block Descriptor Length > 0.
+	               The descriptors come between header and page.
+	*/
+	block_descr_len = c->page->data[6] * 256 + c->page->data[7];
+
+	/* Skip over Mode Data Header and block descriptors */
+	page = c->page->data + 8 + block_descr_len;
 
 	/* ts A61225 :
 	   Although MODE SENSE indeed belongs to SPC, the returned code page
 	   2Ah is part of MMC-1 to MMC-3. In MMC-1 5.2.3.4. it has 22 bytes,
 	   in MMC-3 6.3.11 there are at least 28 bytes plus a variable length
 	   set of speed descriptors. In MMC-5 E.11 it is declared "legacy".
+	   ts B11031 :
+	   qemu emulates an ATAPI DVD-ROM, which delivers only a page length
+	   of 18. This is now tolerated.
 	*/
 	/* ts A90603 :
 	   SPC-1 8.3.3 enumerates mode page format bytes from 0 to n and
@@ -411,8 +416,9 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	if (page_length + 10 > old_alloc_len)
 		page_length = old_alloc_len - 10;
 
-	/* ts A90602 : 20 asserts page[21]. (see SPC-1 8.3.3) */
-	if (page_length < 20) {
+	/* ts A90602 : page_length N asserts page[N+1]. (see SPC-1 8.3.3) */
+	/* ts B11031 : qemu drive has a page_length of 18 */
+	if (page_length < 18) {
 		m->valid = -1;
 		sprintf(msg, "MODE SENSE page 2A too short: %s : %d",
 			d->devname, page_length);
@@ -441,8 +447,11 @@ static int spc_sense_caps_al(struct burn_drive *d, int *alloc_len, int flag)
 	m->max_read_speed = page[8] * 256 + page[9];
 	m->cur_read_speed = page[14] * 256 + page[15];
 
-	m->max_write_speed = page[18] * 256 + page[19];
-	m->cur_write_speed = page[20] * 256 + page[21];
+	m->max_write_speed = m->cur_write_speed = 0;
+	if (page_length >= 18) /* note: page length is page size - 2 */
+		m->max_write_speed = page[18] * 256 + page[19];
+	if (page_length >= 20)
+		m->cur_write_speed = page[20] * 256 + page[21];
 
 	/* ts A61021 : New field to be set by atip (or following MMC-3 info) */
 	m->min_write_speed = m->max_write_speed;
@@ -542,7 +551,7 @@ ex:
 
 void spc_sense_caps(struct burn_drive *d)
 {
-	int alloc_len, start_len = 30, ret;
+	int alloc_len, start_len = 30, minimum_len = 28, ret;
 
 	mmc_start_if_needed(d, 1);
 	if (mmc_function_spy(d, "sense_caps") <= 0)
@@ -555,7 +564,12 @@ void spc_sense_caps(struct burn_drive *d)
 	fprintf(stderr,"LIBBURN_DEBUG: 5Ah alloc_len = %d , ret = %d\n",
 			alloc_len, ret);
 */
-	if (alloc_len >= start_len && ret > 0)
+	/* ts B11103:
+	   qemu ATAPI DVD-ROM delivers only 28.
+	   SanDisk Cruzer U3 memory stick throws error on alloc_len < 30.
+	   MMC-1 prescribes that 30 are available. qemu tolerates 30.
+	*/
+	if (alloc_len >= minimum_len && ret > 0)
 		/* second execution with announced length */
 		spc_sense_caps_al(d, &alloc_len, 0);
 }
@@ -628,7 +642,6 @@ void spc_select_error_params(struct burn_drive *d,
 		c->page->data[10] |= 4;
 	if (!o->hardware_error_recovery)
 		c->page->data[10] |= 1;
-/*burn_print(1, "error parameter 0x%x\n", c->page->data[10]);*/
 	c->page->data[11] = d->params.retries;
 	c->dir = TO_DRIVE;
 	d->issue_command(d, c);
@@ -672,7 +685,6 @@ void spc_sense_write_params(struct burn_drive *d)
 	m = d->mdata;
 	if (!c->error) {
 		page = c->page->data + 8;
-		burn_print(1, "write page length 0x%x\n", page[1]);
 		m->write_page_length = page[1];
 		m->write_page_valid = 1;
 	} else
@@ -763,9 +775,6 @@ void spc_select_write_params(struct burn_drive *d,
 
 	c->page->bytes = alloc_len;
 
-	burn_print(12, "using write page length %d (valid %d)\n",
-		   d->mdata->write_page_length, d->mdata->write_page_valid);
-
 	/* ts A61229 */
 	if (mmc_compose_mode_page_5(d, o, c->page->data + 8) <= 0)
 		goto ex;
@@ -811,9 +820,6 @@ void spc_probe_write_modes(struct burn_drive *d)
 
 	/* ts A70213 : added pseudo try_write_type 4 to set a suitable mode */
 	while (try_write_type != 5) {
-		burn_print(9, "trying %d, %d\n", try_write_type,
-			   try_block_type);
-
 		/* ts A70213 */
 		if (try_write_type == 4) {
 			/* Pseudo write type NONE . Set a useable write mode */
@@ -852,10 +858,9 @@ void spc_probe_write_modes(struct burn_drive *d)
 
 		spc_decode_sense(c->sense, 0, &key, &asc, &ascq);
 		if (key)
-			burn_print(7, "%d not supported\n", try_block_type);
+			/* try_block_type not supported */;
 		else {
-			burn_print(7, "%d:%d SUPPORTED MODE!\n",
-				   try_write_type, try_block_type);
+			/* try_write_type, try_block_type is supported mode */
 			if (try_write_type == 2)	/* sao */
 				d->block_types[try_write_type] =
 					BURN_BLOCK_SAO;
@@ -1036,9 +1041,6 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 	sprintf(msg, "[%X %2.2X %2.2X] ", *key, *asc, *ascq);
 	msg= msg + strlen(msg);
 
-	burn_print(12, "CONDITION: 0x%x 0x%x 0x%x on %s %s\n",
-		   *key, *asc, *ascq, d->idata->vendor, d->idata->product);
-
 	switch (*asc) {
 	case 0x00:
 		if (*key > 0 || *ascq > 0) 
@@ -1334,10 +1336,6 @@ enum response scsi_error(struct burn_drive *d, unsigned char *sense,
 
 	BURN_ALLOC_MEM(msg, char, 160);
 	resp = scsi_error_msg(d, sense, senselen, msg, &key, &asc, &ascq);
-	if (asc == 0 || asc == 0x3A)
-		burn_print(12, "%s\n", msg);
-	else
-		burn_print(1, "%s\n", msg);
 ex:;
 	if (ret == -1)
 		resp = FAIL;
@@ -1410,7 +1408,7 @@ static char *scsi_command_name(unsigned int c, int flag)
         case 0xbe:
 		return "READ CD";
 	}
-	return "(NOT IN COMMAND LIST)";
+	return "(NOT IN LIBBURN COMMAND LIST)";
 }
 
 
@@ -1458,38 +1456,39 @@ ex:;
 }
 
 
-/* ts A91106 */
+/* ts B11110 */
 /* @param flag bit0= do not show eventual data payload sent to the drive
                      (never with WRITE commands)
-               bit1= show write length and target LBA in decimal
 */
-int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
+int scsi_show_command(unsigned char *opcode, int oplen, int dir,
+                      unsigned char *data, int bytes,
+                      void *fp_in, int flag)
 {
 	int i;
 	FILE *fp = fp_in;
 
 	fprintf(fp, "\n%s\n",
-		 scsi_command_name((unsigned int) c->opcode[0], 0));
-	for(i = 0; i < 16 && i < c->oplen; i++)
-		fprintf(fp, "%2.2x ", c->opcode[i]);
+		 scsi_command_name((unsigned int) opcode[0], 0));
+	for(i = 0; i < 16 && i < oplen; i++)
+		fprintf(fp, "%2.2x ", opcode[i]);
 	if (i > 0)
 		fprintf(fp, "\n");
 	if (flag & 1)
 		return 1;
-	if (c->opcode[0] == 0x2A) { /* WRITE 10 */
+	if (opcode[0] == 0x2A) { /* WRITE 10 */
 		if (flag & 2)
 			fprintf(fp, "%d -> %d\n",
-				(c->opcode[7] << 8) | c->opcode[8], 
-				mmc_four_char_to_int(c->opcode + 2));
-	} else if (c->opcode[0] == 0xAA) { /* WRITE 12 */
+				(opcode[7] << 8) | opcode[8], 
+				mmc_four_char_to_int(opcode + 2));
+	} else if (opcode[0] == 0xAA) { /* WRITE 12 */
 		if (flag & 2)
 			fprintf(fp, "%d -> %d\n",
-				mmc_four_char_to_int(c->opcode + 6),
-				mmc_four_char_to_int(c->opcode + 2));	
-	} else if (c->dir == TO_DRIVE) {
-		fprintf(fp, "To drive: %db\n", c->page->bytes);
-		for (i = 0; i < c->page->bytes; i++) 
-			fprintf(fp, "%2.2x%c", c->page->data[i],
+				mmc_four_char_to_int(opcode + 6),
+				mmc_four_char_to_int(opcode + 2));	
+	} else if (dir == TO_DRIVE && !(flag & 1)) {
+		fprintf(fp, "To drive: %db\n", bytes);
+		for (i = 0; i < bytes; i++) 
+			fprintf(fp, "%2.2x%c", data[i],
 				((i % 20) == 19 ? '\n' : ' '));
 		if (i % 20)
 			fprintf(fp, "\n");
@@ -1497,24 +1496,39 @@ int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
 	return 1;
 }
 
+
+
 /* ts A91106 */
-int scsi_show_cmd_reply(struct command *c, void *fp_in, int flag)
+/* @param flag bit0= do not show eventual data payload sent to the drive
+                     (never with WRITE commands)
+*/
+int scsi_show_cmd_text(struct command *c, void *fp_in, int flag)
+{
+	return scsi_show_command(c->opcode, c->oplen, c->dir, c->page->data,
+	                         c->page->bytes, fp_in, flag);
+}
+
+
+/* ts A91106 */ /* ts B11110 */
+int scsi_show_command_reply(unsigned char *opcode, int data_dir,
+                            unsigned char *data, int dxfer_len,
+                            void *fp_in, int flag)
 {
 	int i;
 	FILE *fp = fp_in;
 
-	if (c->dir != FROM_DRIVE)
+	if (data_dir != FROM_DRIVE)
 		return 2;
-	if (c->opcode[0] == 0x28 || c->opcode[0] == 0x3C ||
-	    c->opcode[0] == 0xA8 || c->opcode[0] == 0xBE) {
+	if (opcode[0] == 0x28 || opcode[0] == 0x3C ||
+	    opcode[0] == 0xA8 || opcode[0] == 0xBE) {
 							/* READ commands */
 		/* >>> report amount of data */;
 
 		return 2;
 	}
-	fprintf(fp, "From drive: %db\n", c->dxfer_len);
-	for (i = 0; i < c->dxfer_len; i++)
-		fprintf(fp, "%2.2x%c", c->page->data[i],
+	fprintf(fp, "From drive: %db\n", dxfer_len);
+	for (i = 0; i < dxfer_len; i++)
+		fprintf(fp, "%2.2x%c", data[i],
 			((i % 20) == 19 ? '\n' : ' '));
 	if (i % 20)
 		fprintf(fp, "\n");
@@ -1522,31 +1536,51 @@ int scsi_show_cmd_reply(struct command *c, void *fp_in, int flag)
 }
 
 
+/* ts B11110 */ 
+/** Logs command (before execution) */
+int scsi_log_command(unsigned char *opcode, int oplen, int data_dir,
+                     unsigned char *data, int bytes,
+                     void *fp_in, int flag)
+{
+	FILE *fp = fp_in;
+
+	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
+		scsi_show_command(opcode, oplen, data_dir, data, bytes, fp, 0);
+		if (burn_sg_log_scsi & 4)
+			fflush(fp);
+	}
+	if (fp == stderr || !(burn_sg_log_scsi & 2))
+		return 1;
+	scsi_log_command(opcode, oplen, data_dir, data, bytes, stderr, 0);
+	return 1;
+}
+
+
 /* ts A91218 (former sg_log_cmd ts A70518) */ 
 /** Logs command (before execution) */
 int scsi_log_cmd(struct command *c, void *fp_in, int flag)
 {
-	FILE *fp = fp_in;
+	int ret, bytes = 0;
+	unsigned char *data = NULL;
 
-	if (fp != NULL && (fp == stderr || (burn_sg_log_scsi & 1))) {
-		scsi_show_cmd_text(c, fp, 0);
-		if (burn_sg_log_scsi & 4)
-			fflush(fp);
+	if (c->page != NULL) {
+		data = c->page->data;
+		bytes = c->page->bytes;
 	}
-	if (fp == stderr || !(burn_sg_log_scsi & 2))
-		return 1;
-	scsi_log_cmd(c, stderr, flag);
-	return 1;
+	ret = scsi_log_command(c->opcode, c->oplen, c->dir, data, bytes,
+	                       fp_in, flag);
+	return ret;
 }
 
 
-/* ts A91221 (former sg_log_err ts A91108) */
+/* ts B11110 */
 /** Logs outcome of a sg command.
     @param flag  bit0 causes an error message
                  bit1 do not print duration
 */
-int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
-		 int sense_len, int duration, int flag)
+int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
+                   int dxfer_len, void *fp_in, unsigned char sense[18],
+		   int sense_len, int duration, int flag)
 {
 	char durtxt[20];
 	FILE *fp = fp_in;
@@ -1572,7 +1606,9 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 				(unsigned int) key, (unsigned int) asc,
 				(unsigned int) ascq, durtxt);
 		} else {
-			scsi_show_cmd_reply(c, fp, 0);
+			scsi_show_command_reply(opcode, data_dir, data,
+			                        dxfer_len, fp, 0);
+
 			if (!(flag & 2))
 				fprintf(fp,"%6d ms\n", duration);
 		}
@@ -1581,11 +1617,32 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 	}
 	if (fp == stderr || !(burn_sg_log_scsi & 2))
 		return 1;
-	scsi_log_err(c, stderr, sense, sense_len, duration, flag);
+	scsi_log_reply(opcode, data_dir, data, dxfer_len,
+	               stderr, sense, sense_len, duration, flag);
+
 	return 1;
 }
 
 
+/* ts A91221 (former sg_log_err ts A91108) */
+/** Legacy frontend to scsi_log_reply().
+    @param flag  bit0 causes an error message
+                 bit1 do not print duration
+*/
+int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
+		 int sense_len, int duration, int flag)
+{
+	int ret;
+	unsigned char *data = NULL;
+
+	if (c->page != NULL)
+		data = c->page->data;
+	ret= scsi_log_reply(c->opcode, c->dir, data, c->dxfer_len ,
+	                    fp_in, sense, sense_len, duration, flag);
+	return ret;
+}
+
+
 /* ts B00808 */
 /*
     @param flag    bit0 = do not retry
