@@ -1,11 +1,12 @@
 
 /*  test/telltoc.c , API illustration of obtaining media status info */
-/*  Copyright (C) 2006 Thomas Schmitt <scdbackup@gmx.net> Provided under GPL */
+/*  Copyright (C) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net> 
+    Provided under GPL */
 
 /**                               Overview 
   
   telltoc is a minimal demo application for the library libburn as provided
-  on  http://libburnia.pykix.org . It can list the available devices, can
+  on  http://libburnia-project.org . It can list the available devices, can
   display some drive properties, the type of media, eventual table of content
   and multisession info for mkisofs option -C . 
   It's main purpose, nevertheless, is to show you how to use libburn and also
@@ -24,6 +25,8 @@
      telltoc_media()   prints some information about the media in a drive
      telltoc_toc()     prints a table of content (if there is content)
      telltoc_msinfo()  prints parameters for mkisofs option -C
+     telltoc_read_and_print()  reads from data CD or from DVD and prints 7-bit 
+                       to stdout (encodings 0,2) or 8-bit to file (encoding 1)
   When everything is done, main() releases the drive and shuts down libburn:
      burn_drive_release();
      burn_finish()
@@ -72,6 +75,11 @@ int telltoc_aquire_by_adr(char *drive_adr);
 int telltoc_aquire_by_driveno(int *drive_no, int silent);
 
 
+/* A message from --toc to --read_and_print (CD tracksize is a bit tricky) */
+static int last_track_start = 0, last_track_size = -1;
+static int media_is_cd_profile = 0;
+
+
 /* ------------------------------- API gestures ---------------------------- */
 
 /** You need to aquire a drive before burning. The API offers this as one
@@ -100,22 +108,33 @@ int telltoc_aquire_drive(char *drive_adr, int *driveno, int silent_drive)
 /** If the persistent drive address is known, then this approach is much
     more un-obtrusive to the systemwide livestock of drives. Only the
     given drive device will be opened during this procedure.
+    Special drive addresses stdio:<path> direct output to a hard disk file
+    which will behave much like a DVD-RAM.
 */
 int telltoc_aquire_by_adr(char *drive_adr)
 {
 	int ret;
-	
 	char libburn_drive_adr[BURN_DRIVE_ADR_LEN];
 
+	/* <<< ts A70907 FOR TESTING ONLY !
+	struct burn_drive_info *test_drive_list;
+	*/
+
 	/* This tries to resolve links or alternative device files */
 	ret = burn_drive_convert_fs_adr(drive_adr, libburn_drive_adr);	
 	if (ret<=0) {
-		fprintf(stderr,"Address does not lead to a CD burner: '%s'\n",
+		fprintf(stderr, "Address does not lead to a CD burner: '%s'\n",
 			drive_adr);
-		return ret;
+		return 0;
 	}
-	fprintf(stderr,"Aquiring drive '%s' ...\n",libburn_drive_adr);
-	ret = burn_drive_scan_and_grab(&drive_list,libburn_drive_adr,1);
+
+	/* <<< ts A70907 FOR TESTING ONLY ! 
+	ret = burn_drive_scan_and_grab(&test_drive_list, "/dev/sg2", 1);
+	*/
+
+	fprintf(stderr,"Aquiring drive '%s' ...\n", libburn_drive_adr);
+	ret = burn_drive_scan_and_grab(&drive_list, libburn_drive_adr, 1);
+
 	if (ret <= 0) {
 		fprintf(stderr,"FAILURE with persistent drive address  '%s'\n",
 			libburn_drive_adr);
@@ -123,6 +142,11 @@ int telltoc_aquire_by_adr(char *drive_adr)
 		fprintf(stderr,"Done\n");
 		drive_is_grabbed = 1;
 	}
+
+	/* <<< ts A70907 FOR TESTING ONLY !
+	burn_drive_info_free(test_drive_list);
+	*/
+
 	return ret;
 }
 
@@ -147,14 +171,14 @@ int telltoc_aquire_by_driveno(int *driveno, int silent_drive)
 
 	fprintf(stderr, "Beginning to scan for devices ...\n");
 	while (!burn_drive_scan(&drive_list, &drive_count))
-		usleep(1002);
+		usleep(100002);
 	if (drive_count <= 0 && *driveno >= 0) {
 		fprintf(stderr, "FAILED (no drives found)\n");
 		return 0;
 	}
 	fprintf(stderr, "Done\n");
 
-	for (i = 0; i < drive_count; i++) {
+	for (i = 0; i < (int) drive_count; i++) {
 		if (*driveno >= 0 && (silent_drive || *driveno != i))
 	continue;
 		if (burn_drive_get_adr(&(drive_list[i]), adr) <=0)
@@ -173,7 +197,7 @@ int telltoc_aquire_by_driveno(int *driveno, int silent_drive)
 	/* We already made our choice via command line. (default is 0)
 	   So we just have to keep our desired drive and drop all others.
 	 */
-	if (drive_count <= *driveno) {
+	if ((int) drive_count <= *driveno) {
 		fprintf(stderr,
 			"Found only %d drives. Number %d not available.\n",
 			drive_count, *driveno);
@@ -181,7 +205,7 @@ int telltoc_aquire_by_driveno(int *driveno, int silent_drive)
 	}
 
 	/* Drop all drives which we do not want to use */
-	for (i = 0; i < drive_count; i++) {
+	for (i = 0; i < (int) drive_count; i++) {
 		if (i == *driveno) /* the one drive we want to keep */
 	continue;
 		ret = burn_drive_info_forget(&(drive_list[i]),0);
@@ -234,7 +258,7 @@ int telltoc_media(struct burn_drive *drive)
 
 	printf("Media current: ");
 	ret = burn_disc_get_profile(drive, &profile_no, profile_name);
-	if (profile_no > 0 && ret >0) {
+	if (profile_no > 0 && ret > 0) {
 		if (profile_name[0])
 			printf("%s\n", profile_name);
 		else
@@ -443,8 +467,9 @@ int telltoc_formatlist(struct burn_drive *drive)
 int telltoc_toc(struct burn_drive *drive)
 {
 	int num_sessions = 0 , num_tracks = 0 , lba = 0, pmin, psec, pframe;
-	int track_count = 0;
+	int track_count = 0, pno;
 	int session_no, track_no;
+	char profile_name[80];
 	struct burn_disc *disc= NULL;
 	struct burn_session **sessions;
 	struct burn_track **tracks;
@@ -479,6 +504,7 @@ int telltoc_toc(struct burn_drive *drive)
 				track_count,
 				((toc_entry.control&7)<4?"audio":"data "),
 				lba, pmin, psec, pframe);
+			last_track_start = lba;
 		}
 		burn_session_get_leadout_entry(sessions[session_no],
 						&toc_entry);
@@ -494,6 +520,11 @@ int telltoc_toc(struct burn_drive *drive)
 		printf("Media content: session %2d  ", session_no+1);
 		printf("leadout            lba: %9d  %4.2d:%2.2d:%2.2d\n",
 			lba, pmin, psec, pframe);
+		last_track_size = lba - last_track_start;
+		if (burn_disc_get_profile(drive, &pno, profile_name) > 0)
+			if (pno == 0x09 || pno == 0x0a)
+				media_is_cd_profile = 1;
+			
 	}
 	if (disc!=NULL)
 		burn_disc_free(disc);
@@ -558,6 +589,125 @@ ex:;
 }
 
 
+/**
+  @param encoding determins how to format output on stdout:
+                  0 = default , 1 = raw 8 bit (dangerous for tty) , 2 = hex
+*/
+int telltoc_read_and_print(struct burn_drive *drive, 
+	int start_sector, int sector_count, char *raw_file, int encoding)
+{
+	int j, i, request = 16, done, lbas = 0, final_cd_try = -1, todo;
+	int ret = 0;
+	char buf[16 * 2048], line[81];
+	off_t data_count, total_count= 0, last_reported_count= 0;
+	struct stat stbuf;
+	FILE *raw_fp = NULL;
+
+	if (start_sector == -1)
+		start_sector = last_track_start;
+	if (sector_count == -1) {
+		sector_count = last_track_start + last_track_size
+				- start_sector;
+		if (media_is_cd_profile)  /* In case it is a TAO track */
+			final_cd_try = 0; /* allow it (-1 is denial) */
+	}
+	if (start_sector < 0)
+		start_sector = 0;
+	if (sector_count <= 0)
+		sector_count = 2147483632;
+
+	if (sector_count <= 0)
+		return -1;
+	if (encoding == 1) {
+                if (stat(raw_file,&stbuf) != -1) {
+			if (!(S_ISCHR(stbuf.st_mode) || S_ISFIFO(stbuf.st_mode)
+				 || (stbuf.st_mode & S_IFMT) == S_IFSOCK )) {
+                   	 	fprintf(stderr,
+				  "SORRY: target file '%s' already existing\n",
+				  raw_file);
+				return 1;
+			}
+                }
+                raw_fp = fopen(raw_file,"w");
+                if (raw_fp == NULL) {
+                    fprintf(stderr,"SORRY: cannot open target file '%s' (%s)\n", raw_file, strerror(errno));
+                    return 1;
+                }
+		printf(
+	"Data         : start=%ds , count=%ds , read=0s , encoding=%d:'%s'\n",
+		start_sector, sector_count, encoding, raw_file);
+	} else 
+		printf(
+		"Data         : start=%ds , count=%ds , read=0 , encoding=%d\n",
+		start_sector, sector_count, encoding);
+	todo = sector_count - 2*(final_cd_try > -1);
+	for (done = 0; done < todo && final_cd_try != 1; done += request) {
+		if (todo - done > 16)
+			request = 16;
+		else
+			request = todo - done;
+		ret = burn_read_data(drive,
+			((off_t) start_sector + done) * (off_t) 2048,
+			buf, (off_t) (request * 2048), &data_count, 1);
+			
+print_result:;
+		total_count += data_count;
+		if (encoding == 1) {
+			if (data_count > 0)
+				fwrite(buf, data_count, 1, raw_fp);
+		} else for (i = 0; i < data_count; i += 16) {
+			if (encoding == 0) {
+				sprintf(line, "%8ds + %4d : ",
+					start_sector + done + i / 2048,
+					i % 2048);
+				lbas = strlen(line);
+			}
+			for (j = 0; j < 16 && i + j < data_count; j++) {
+				if (buf[i + j] >= ' ' && buf[i + j] <= 126 &&
+					encoding != 2)
+					sprintf(line + lbas + 3 * j, " %c ",
+					 	(int) buf[i + j]);
+				else
+					sprintf(line + lbas + 3 * j, "%2.2X ",
+					 	(unsigned char) buf[i + j]);
+			}
+			line[lbas + 3 * (j - 1) + 2] = 0;
+			printf("%s\n",line);
+		}
+		if (encoding == 1 &&
+		    total_count - last_reported_count >= 1000 * 2048) {
+			fprintf(stderr, 
+	     		 "\rReading data : start=%ds , count=%ds , read=%ds  ",
+			 start_sector, sector_count,
+			  (int) (total_count / (off_t) 2048));
+			last_reported_count = total_count;
+		}
+		if (ret <= 0) {
+			fprintf(stderr, "SORRY : Reading failed.\n");
+	break;
+		}
+	}
+	if (ret > 0 && media_is_cd_profile && final_cd_try == 0) {
+		/* In a SAO track the last 2 frames should be data too */
+		final_cd_try = 1;
+		burn_read_data(drive,
+			((off_t) start_sector + todo) * (off_t) 2048,
+			buf, (off_t) (2 * 2048), &data_count, 2);
+		if (data_count < 2 * 2048)
+			fprintf(stderr, "\rNOTE : Last two frames of CD track unreadable. This is normal if TAO track.\n");
+		if (data_count > 0)
+			goto print_result;	
+	}
+	if (last_reported_count > 0)
+		fprintf(stderr,
+"\r                                                                       \r");
+	printf("End Of Data  : start=%ds , count=%ds , read=%ds\n",
+		start_sector, sector_count,(int) (total_count / (off_t) 2048));
+
+	return ret;
+}
+
+
 /** The setup parameters of telltoc */
 static char drive_adr[BURN_DRIVE_ADR_LEN] = {""};
 static int driveno = 0;
@@ -566,6 +716,8 @@ static int do_toc = 0;
 static int do_msinfo = 0;
 static int print_help = 0;
 static int do_capacities = 0;
+static int read_start = -2, read_count = -2, print_encoding = 0;
+static char print_raw_file[4096] = {""};
 
 
 /** Converts command line arguments into above setup parameters.
@@ -608,6 +760,26 @@ int telltoc_setup(int argc, char **argv)
         } else if (!strcmp(argv[i], "--toc")) {
 	    do_toc = 1;
 
+        } else if (!strcmp(argv[i], "--read_and_print")) {
+            i+= 3;
+            if (i >= argc) {
+                fprintf(stderr,"--read_and_print requires three arguments: start count encoding(try 0, not 1)\n");
+                return 1;
+            }
+            sscanf(argv[i-2], "%d", &read_start);
+            sscanf(argv[i-1], "%d", &read_count);
+            print_encoding = 0;
+            if(strncmp(argv[i], "raw:", 4) == 0 || strcmp(argv[i],"1:") == 0) {
+                print_encoding = 1;
+                strcpy(print_raw_file, strchr(argv[i], ':') + 1);
+		if (strcmp(print_raw_file, "-") == 0) {
+                	fprintf(stderr,
+		"--read_and_print does not write to \"-\" as stdout.\n");
+                	return 1;
+		}
+            } else if(strcmp(argv[i], "hex") == 0 || strcmp(argv[i], "2") == 0)
+               print_encoding = 2;
+            
         } else if (!strcmp(argv[i], "--help")) {
             print_help = 1;
 
@@ -622,6 +794,7 @@ int telltoc_setup(int argc, char **argv)
         printf("Usage: %s\n", argv[0]);
         printf("       [--drive <address>|<driveno>|\"-\"]\n");
         printf("       [--media]  [--capacities]  [--toc]  [--msinfo]\n");
+        printf("       [--read_and_print <start> <count> \"0\"|\"hex\"|\"raw\":<path>]\n");
         printf("Examples\n");
         printf("A bus scan (needs rw-permissions to see a drive):\n");
         printf("  %s --drive -\n",argv[0]);
@@ -635,6 +808,12 @@ int telltoc_setup(int argc, char **argv)
         printf("  mkisofs ... -C \"$msinfo\" ...\n");
 	printf("Obtain what is available about drive 0 and its media\n");
 	printf("  %s --drive 0\n",argv[0]);
+	printf("View blocks 16 to 19 of data CD or DVD in human readable form\n");
+	printf("  %s --drive /dev/sr1 --read_and_print 16 4 0 | less\n",
+		argv[0]);
+        printf("Copy last data track from CD to file /tmp/data\n");
+        printf("  %s --drive /dev/sr1 --toc --read_and_print -1 -1 raw:/tmp/data\n",
+                argv[0]);
     }
     return 0;
 }
@@ -656,14 +835,14 @@ int main(int argc, char **argv)
 			msinfo_alone = 1;
 	}
 	/* Default option is to do everything if possible */
-    	if (do_media==0 && do_msinfo==0 && do_capacities==0 && do_toc==0 
-	    && driveno!=-1) {
+    	if (do_media==0 && do_msinfo==0 && do_capacities==0 && do_toc==0 &&
+		(read_start < 0 || read_count <= 0) && driveno!=-1) {
 		if(print_help)
 			exit(0);
-		full_default = do_media = do_msinfo = do_capacities= do_toc = 1;
+		full_default = do_media = do_msinfo = do_capacities= do_toc= 1;
 	}
 
-	fprintf(stderr, "Initializing libburnia.pykix.org ...\n");
+	fprintf(stderr, "Initializing libburnia-project.org ...\n");
 	if (burn_initialize())
 		fprintf(stderr, "Done\n");
 	else {
@@ -671,8 +850,11 @@ int main(int argc, char **argv)
 		exit(33);
 	}
 
-	/* Print messages of severity SORRY or more directly to stderr */
-	burn_msgs_set_severities("NEVER", "SORRY", "telltoc : ");
+	/* Print messages of severity WARNING or more directly to stderr */
+	burn_msgs_set_severities("NEVER", "WARNING", "telltoc : ");
+
+	/* Activate the default signal handler */
+	burn_set_signal_handling("telltoc : ", NULL, 0);
 
 	/** Note: driveno might change its value in this call */
 	ret = telltoc_aquire_drive(drive_adr, &driveno, !full_default);
@@ -709,6 +891,13 @@ int main(int argc, char **argv)
 		if (ret<=0)
 			{ret = 38; goto release_drive; }
 	}
+	if (read_start >= -1 && (read_count > 0 || read_count == -1)) {
+		ret = telltoc_read_and_print(drive_list[driveno].drive,
+				read_start, read_count, print_raw_file,
+				print_encoding);
+		if (ret<=0)
+			{ret = 40; goto release_drive; }
+	}
 
 	ret = 0;
 	if (toc_failed)
